<?php

use Phinx\Migration\AbstractMigration;
use Phinx\Db\Adapter\MysqlAdapter;

class AddAds extends AbstractMigration
{
    public function change()
    {
        $this
            ->table("ads_emplacement", ['id' => false, 'primary_key' => ["ads_emplacement_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('ads_emplacement_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('ads_emplacement_nom', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'ads_emplacement_id'])
            ->create();

        $this
            ->table("ads", ['id' => false, 'primary_key' => ["ads_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('ads_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('ads_date_ajout', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'ads_id'])
            ->addColumn('ads_date_modif', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'ads_date_ajout'])
            ->addColumn('ads_date_debut', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'ads_date_modif'])
            ->addColumn('ads_date_fin', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'ads_date_debut'])
            ->addColumn('ads_emplacement_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_date_fin', 'signed' => false])
            ->addColumn('categorie_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_emplacement_id', 'signed' => false])
            ->addColumn('actif_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'categorie_id', 'signed' => false])
            ->addColumn('ads_rang', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'actif_id'])
            ->addForeignKey('actif_id', 'actif', 'actif_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('ads_emplacement_id', 'ads_emplacement', 'ads_emplacement_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('categorie_id', 'categorie', 'categorie_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['actif_id'], ['name' => "actif_id", 'unique' => false])
            ->addIndex(['ads_emplacement_id'], ['name' => "ads_emplacement_id", 'unique' => false])
            ->addIndex(['ads_date_debut', 'ads_date_fin'], ['name' => "ads_date_debut_2", 'unique' => false])
            ->addIndex(['categorie_id'], ['name' => "categorie_id", 'unique' => false])
            ->create();

        $this
            ->table("ads_boutique", ['id' => false, 'primary_key' => ["ads_boutique_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('ads_boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('ads_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_boutique_id', 'signed' => false])
            ->addColumn('boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_id', 'signed' => false])
            ->addColumn('ads_boutique_statut_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'boutique_id'])
            ->addForeignKey('ads_id', 'ads', 'ads_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('boutique_id', 'boutique', 'boutique_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['ads_id', 'boutique_id'], ['name' => "ads_id_2", 'unique' => true])
            ->addIndex(['ads_id'], ['name' => "ads_id", 'unique' => false])
            ->addIndex(['boutique_id'], ['name' => "boutique_id", 'unique' => false])
            ->addIndex(['ads_boutique_statut_id'], ['name' => "ads_boutique_statut_id", 'unique' => false])
            ->create();

        $this
            ->table("ads_nom", ['id' => false, 'primary_key' => ["ads_nom_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('ads_nom_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('ads_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_nom_id', 'signed' => false])
            ->addColumn('langue_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'ads_id', 'signed' => false])
            ->addColumn('ads_nom_nom', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'langue_id'])
            ->addColumn('ads_nom_texte', 'text', ['null' => false, 'limit' => 65535, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'ads_nom_nom'])
            ->addColumn('ads_nom_image', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'ads_nom_texte'])
            ->addColumn('ads_nom_image_logo', 'string', ['null' => false, 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'ads_nom_image'])
            ->addForeignKey('langue_id', 'langue', 'langue_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('ads_id', 'ads', 'ads_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['langue_id', 'ads_id'], ['name' => "langue_id_2", 'unique' => true])
            ->addIndex(['langue_id'], ['name' => "langue_id", 'unique' => false])
            ->addIndex(['ads_id'], ['name' => "ads_id", 'unique' => false])
            ->create();
    }
}
