<?php

use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Catalogue
 *
 * Exemple utilisation en CLI :
 *  - Produit : php dedi import:catalogue produit import/vin-20150316.xlsx vin
 *  - Categorie : php dedi import:catalogue categories import/Categorie.xls Menu
 *
 */
class Catalogue
{
    /**
     * @var SiteSQL
     */
    protected $sql;

    /**
     * @var Query
     */
    protected $query;

    protected $output;

    protected $couleur;
    protected $appellation;
    protected $etat;
    protected $format;
    protected $classification;

    /**
     * @var array $tables
     */
    protected static $tables = array(
        'products' => array(
            'produit', 'produit_nom', 'produit_attribut', 'produit_boutique', 'produit_caracteristique_valeur',
            'produit_categorie', 'produit_description', 'produit_description_nom', 'produit_gamme', 'produit_image',
            'produit_list', 'produit_list_produit', 'produit_option', 'produit_option_nom', 'produit_option_valeur',
            'produit_option_valeur_nom', 'produit_stock'
        ),
        'orders' => array('commande', 'commande_produit', 'commande_produit_attribut', 'avoir'),
        'manufacturers' => array('fournisseur', 'fournisseur_nom', 'marque', 'marque_nom'),
        'features' => array('caracteristique_valeur', 'caracteristique_valeur_nom', 'produit_caracteristique_valeur'),
        'descriptions' => array('produit_description', 'produit_description_nom'),
        'customers' => array('client', 'client_role'),
        'addresses' => array('adresse')
    );

    /**
     * @var array $tablesJoin
     */
    protected static $tablesJoin = array(
        'product' => array('orders', 'manufacturers', 'features', 'products', 'descriptions'),
        'customer' => array('customers', 'addresses')
    );

    public function __construct(OutputInterface $output = null)
    {
        $this->sql = SiteSQL::getInstance();
        $this->query = new Query();
        $this->output = $output;

    }

    /**
     * fonction pour importer des données depuis une feuille d'un fichier Excel

     * LE CODE DE CETTE FONCTION N'AS PAS DE RAISON D'ETRE MODIFIE
     *
     * @param string $method nom de la methode a appele une fois les données recuperer dans un tableau (pour traiter / insrer les data en BDD)
     * @param PHPExcel_Worksheet $sheet
     * @param array $formvars
     *
     * @throws PHPExcel_Calculation_Exception
     * @return void
     */
    public function import($method, PHPExcel_Worksheet $sheet, $formvars = array())
    {
        if(empty($formvars['startData'])) {
            $formvars['startData'] = 0;
        }

        $lines = $sheet->rangeToArray('A1:' . $sheet->getHighestColumn() . $sheet->getHighestRow(), NULL, TRUE, FALSE);

        $this->$method($lines, $formvars);
    }

    /**
     * Fonction pour cleaner les data d'une / plusieurs tables
     *
     * @param array $tables liste des tables pour lesquelles on doit cleaner les data
     *
     * @return void
     */
    public function clearBdd($tables = array())
    {
        $sql = "SET FOREIGN_KEY_CHECKS = 0";
        $this->sql->query($sql);

        foreach ($tables as $table) {
            $sql = "TRUNCATE TABLE " . $table;
            $this->sql->query($sql);
        }
    }

    /**
     * php dedi import:catalogue categories import/Categorie.xls Menu
     *
     * @param $lines
     * @param array $formvars
     *
     * @throws DediException
     * @throws Exception
     */
    public function categories($lines, $formvars = array())
    {
        // on se place dans la boutique 1
        $boutique = Boutique::loadById(1);
        $categorie = new Categorie();

        $i = 0;

        $tablesCategorie = array('categorie', 'categorie_nom', 'categorie_boutique', 'produit_categorie');

        $tables = array_merge($tablesCategorie, array());

        $this->clearBdd($tables);

        echo 'debut import categorie' . "\n";

        foreach ($lines as $line) {
            $i++;
            if ($i == 1) {
                continue;
            }
            $line[2] = trim($line[2]);
            if (empty($line[2])) {
                continue;
            }

            $result = $categorie->get(array('categorie_id' => $line[0]));
            if (empty($result)) {
                $data = array('langue_id' => 1);

                $this->query->reset()->insert('categorie')
                    ->value('categorie_id', $line[0])
                    ->value('categorie_rang', $i*10)
                    ->value('categorie_visible', 1);
                if (!empty($line[1])) {
                    $this->query->value('parent_id', $line[1]);
                }
                $this->sql->query($this->query->build());
                $data['categorie_id'] = $this->sql->db->insert_id;

                if (!empty($data['categorie_id'])) {
                    $this->write('ajout categorie ' . $line[2]);

                    $boutique->createStatut('categorie', $data['categorie_id']);

                    $data['categorie_nom_nom'] = $line[2];
                    $this->sql->save('categorie_nom', $data);

                } else {
                    $this->write('error categorie ' . $line[2], 'error');
                }
            }
        }

        $allCategories = $categorie->get();
        $this->write('ext id');
        foreach ($allCategories as $cat) {
            $categorie->updateExtId($cat['categorie_id']);
        }
    }

    public function produitCategorie($line, $data)
    {
        $categories = explode('|', $line[15]);

        foreach ($categories as $categorie) {
            // recup id categorie auquel est associé le produit
            $this->query->reset()->select()->from('millesimes.categorie')
                ->where('nom', $categorie);
            $result = $this->sql->getOne($this->query->build());

            if (!empty($result)) {
                // recup id menu auquel est associé la catégorie trouvé
                $this->query->reset()->select()->from('millesimes.menu_deroulant2')
                    ->where("FIND_IN_SET(" . $result['id'] . ",md.cat_ids)");
                $results = $this->sql->getAll($this->query->build());

                if (!empty($results)) {
                    // association menu (categorie new system) avec le produit
                    foreach ($results as $res) {
                        $this->sql->save('produit_categorie', array(
                            'produit_id' => $data['produit_id'],
                            'categorie_id' => $res['id']
                        ));
                    }
                }
            }
        }

    }

    public function description($line, $data)
    {
        // TODO : revoir => faire un table pour les titres / types
        return false;
        // handle description and save in produit_description

        $descs = explode('<br />', $line[27]);

        $dataDesc = array(
            'produit_id' => $data['produit_id'],
            'langue_id' => 1,
        );

        $this->query->reset()->select()->from('produit_description')
            ->innerJoin('produit_description_nom', 'AND', array('pd.produit_description_id', 'pdn.produit_description_id'), array('pdn.langue_id', $dataDesc['langue_id']))
            ->where('pd.produit_id', $data['produit_id'])
            ->where('pdn.langue_id', $dataDesc['langue_id']);

        $result = $this->sql->getAll($this->query->build());

        if (empty($result)) {
            $dataDesc['produit_description_id'] = $this->sql->save('produit_description', $dataDesc);
        } else {
            $dataDesc['produit_description_id'] = $result[0]['produit_description_id'];
        }

        foreach ($descs as $desc) {
            list($dataDesc['produit_description_nom_titre'], $dataDesc['produit_description_nom_nom']) = explode(' : ', $desc, 2);
            foreach ($result as $res) {
                if ($res['produit_description_nom_titre'] == $dataDesc['produit_description_nom_titre']) {
                    $dataDesc['produit_description_nom_id'] = $res['produit_description_nom_id'];
                } else {
                    $dataDesc['produit_description_nom_id'] = null;
                }
            }
            $this->sql->save('produit_description_nom', $dataDesc);
        }

    }

    /**
     * @param $caraId
     * @param $value
     * @param $data
     */
    public function caracteristique($caraId, $value, $data)
    {

        $value = $this->sql->db->real_escape_string($value);

        $this->query->reset()->select()->from('caracteristique_valeur')
            ->innerJoin('caracteristique_valeur_nom', 'AND', array('cv.caracteristique_valeur_id', 'cvn.caracteristique_valeur_id'), array('cvn.langue_id', 1))
            ->where('caracteristique_id', $caraId)
            ->where('caracteristique_valeur_nom_nom', $value);

        $result = $this->sql->getOne($this->query->build());

        $dataCara = array(
            'langue_id' => 1,
            'caracteristique_valeur_nom_nom' => $value,
        );
        if (empty($result)) {
            $dataCara['caracteristique_valeur_id'] = $this->sql->save('caracteristique_valeur', array(
                'caracteristique_id' => $caraId,
                'caracteristique_valeur_statut_id' => 1,
                'caracteristique_valeur_visible_filtre' => 1,
            ));
            $this->sql->save('caracteristique_valeur_nom', $dataCara);
        } else {
            $dataCara['caracteristique_valeur_id'] = $result['caracteristique_valeur_id'];
        }

        // assoc produit / cara
        $this->sql->save('produit_caracteristique_valeur', array(
            'caracteristique_valeur_id' => $dataCara['caracteristique_valeur_id'],
            'produit_id' => $data['produit_id']
        ));

    }

    public function searchInfosProduit($info, $value, $opt = array())
    {
        if (!in_array($info, array('couleur', 'appellation', 'etat', 'format', 'classification'))) {
            return false;
        }

        if (empty($this->$info)) {
            $method = 'load' . ucfirst($info);
            $this->$method();
        }

        $value = trim($value);

        $search = array_search($value, $this->$info);

        if ($search === false) {
            $method = 'create' . ucfirst($info);
            $search = $this->$method($value, $opt);
        }

        return $search;
    }

    protected function loadCouleur()
    {
        $this->query->reset()->select()->from('produit_couleur');
        $result = $this->sql->getAll($this->query->build());
        $this->couleur = Tools::formatAssoc($result, 'produit_couleur_id', 'produit_couleur_code');
    }

    protected function createCouleur($value, $opt)
    {
        $data = array(
            'produit_couleur_code' => $value,
            'langue_id' => 1,
            'produit_couleur_nom_nom' => $opt['label']
        );
        $data['produit_couleur_id'] = $this->sql->save('produit_couleur', $data);
        $this->sql->save('produit_couleur_nom', $data);

        $this->couleur[$data['produit_couleur_id']] = $value;
        return $data['produit_couleur_id'];
    }

    protected function loadEtat()
    {
        $this->query->reset()->select()->from('produit_etat');
        $result = $this->sql->getAll($this->query->build());
        $this->etat = Tools::formatAssoc($result, 'produit_etat_id', 'produit_etat_code');
    }

    protected function createEtat($value, $opt)
    {
        $data = array(
            'produit_etat_code' => $value,
            'langue_id' => 1,
            'produit_etat_nom_nom' => $opt['label']
        );
        $data['produit_etat_id'] = $this->sql->save('produit_etat', $data);
        $this->sql->save('produit_etat_nom', $data);

        $this->etat[$data['produit_etat_id']] = $value;
        return $data['produit_etat_id'];
    }

    protected function loadFormat()
    {
        $this->query->reset()->select()->from('produit_format');
        $result = $this->sql->getAll($this->query->build());
        $this->format = Tools::formatAssoc($result, 'produit_format_id', 'produit_format_code');
    }

    protected function createFormat($value, $opt)
    {
        $data = array(
            'produit_format_code' => $value,
            'langue_id' => 1,
            'produit_format_nom_nom' => $opt['label'],
            'produit_format_cl' => $opt['cl']
        );
        $data['produit_format_id'] = $this->sql->save('produit_format', $data);
        $this->sql->save('produit_format_nom', $data);

        $this->format[$data['produit_format_id']] = $value;
        return $data['produit_format_id'];
    }

    protected function loadAppellation()
    {
        $this->query->reset()->select()->from('produit_appellation')
            ->innerJoin('produit_appellation_nom', 'AND', array('pa.produit_appellation_id', 'pan.produit_appellation_id'), array('pan.langue_id', 1));
        $result = $this->sql->getAll($this->query->build());
        $this->appellation = Tools::formatAssoc($result, 'produit_appellation_id', 'produit_appellation_nom_nom');
    }

    protected function createAppellation($value, $opt)
    {
        $data = array(
            'langue_id' => 1,
            'produit_appellation_rang' => 10,
            'produit_appellation_nom_nom' => $value
        );
        $data['produit_appellation_id'] = $this->sql->save('produit_appellation', $data);
        $this->sql->save('produit_appellation_nom', $data);

        $this->appellation[$data['produit_appellation_id']] = $value;
        return $data['produit_appellation_id'];
    }

    protected function loadClassification()
    {
        $this->query->reset()->select()->from('produit_classification')
            ->innerJoin('produit_classification_nom', 'AND', array('pc.produit_classification_id', 'pcn.produit_classification_id'), array('pcn.langue_id', 1));
        $result = $this->sql->getAll($this->query->build());
        $this->classification = Tools::formatAssoc($result, 'produit_classification_id', 'produit_classification_nom_nom');
    }

    protected function createClassification($value, $opt)
    {
        $data = array(
            'langue_id' => 1,
            'produit_classification_rang' => 10,
            'produit_classification_nom_nom' => $value
        );
        $data['produit_classification_id'] = $this->sql->save('produit_classification', $data);
        $this->sql->save('produit_classification_nom', $data);

        $this->classification[$data['produit_classification_id']] = $value;
        return $data['produit_classification_id'];
    }

    /**
     * permet d'identifier de facon unique un produit
     *
     * @param $line
     *
     * @return array
     */
    public function matchProduct($line)
    {
        // TODO : match produit avec les infos qui permettent d'identifier de facon unique un produit

        // critere pour matcher :
        // TODO - Acquit ou CRD
        // - Etat de la bouteille => col 10
        // - Format (bouteille / magnum / etc...) => col 9
        // - Nom => col 1
        // - Couleur =>  col 14
        // - Millesime => col 12
        // - Appellation => col 2
        // - Classification => col 3
        // - Si col 2 et 3 vide => prendre col 4 : explode par espace, [0] => col 3 | [1] => col 2


        $etat = $this->searchInfosProduit('etat', $line[10], array('label' => $line[11]));
        $format = $this->searchInfosProduit('format', $line[9], array('label' => $line[6], 'cl' => $line[8]));
        $couleur = $this->searchInfosProduit('couleur', $line[14], array('label' => $line[13]));
        if (empty($line[2]) && empty($line[3]) && !empty($line[4])) {
            $temp = explode(' ', $line[4]);
            $line[2] = array_pop($temp);
            $line[3] = implode(' ', $temp);
        }
        $appellation = $this->searchInfosProduit('appellation', $line[2]);
        $classification = $this->searchInfosProduit('classification', $line[3]);

        $this->query->reset()->select()
            ->from('produit')
            ->innerJoin('produit_nom', 'AND', array('p.produit_id', 'pn.produit_id'), array('pn.langue_id', 1))
            ->where('produit_etat_id', $etat)
            ->where('produit_couleur_id', $couleur)
            ->where('produit_format_id', $format)
            ->where('produit_appellation_id', $appellation)
            ->where('produit_classification_id', $classification)
            ->where('produit_millesime', $line[12])
            ->where('produit_nom_nom', $this->sql->db->real_escape_string($line[1]));

        // echo $this->query->build() . "\n";

        return $this->sql->getOne($this->query->build());
    }

    /**
     * Permet d'identifier un attribut de facon unique
     *
     * @param $line
     * @param $data
     *
     * @return array
     *
     * @throws Exception
     */
    public function matchAttribut($line, $data)
    {

        $this->query->reset()->select()->from('produit_attribut')
            ->where('produit_id', $data['produit_id'])
            ->where('produit_attribut_conditionnement', $line[20]);

        return $this->sql->getOne($this->query->build());
    }


    public function produit($lines, $formvars = array())
    {
        // on se place dans la boutique 1
        $boutique = Boutique::loadById(1);
        $stock = new Stock();

        $i = 0;

        $tablesProduit = array(
            'produit', 'produit_nom', 'produit_attribut', 'produit_boutique', 'produit_caracteristique_valeur', 'produit_categorie',
            'produit_description', 'produit_description_nom', 'produit_gamme', 'produit_image', 'produit_list', 'produit_list_produit',
            'produit_option', 'produit_option_nom', 'produit_option_valeur', 'produit_option_valeur_nom', 'produit_stock',
            'produit_format', 'produit_format_nom', 'produit_appellation', 'produit_appellation_nom', 'produit_classification', 'produit_classification_nom',
            'produit_etat', 'produit_etat_nom'
        );

        $tablesCommande = array('commande', 'commande_produit', 'commande_produit_attribut');

        $tablesMrqFrs = array('fournisseur', 'fournisseur_nom', 'marque', 'marque_nom');

        $tablesCara = array('caracteristique_valeur', 'caracteristique_valeur_nom', 'produit_caracteristique_valeur');
        $tabesDesc = array('produit_description', 'produit_description_nom');

        // $tables = array_merge($tablesCommande, $tablesProduit, $tablesMrqFrs);
        $tables = array_merge($tablesCara, $tabesDesc);

        // $this->clearBdd($tables);

        echo 'debut import produit' . "\n";
        
        foreach ($lines as $line) {
            $i++;
            if ($i == 1) {
                continue;
            }

            $data = array();

            $taxeId = 7;

            $data['produit_nom_nom'] = $line[1];

            if (empty($line[2]) && empty($line[3]) && !empty($line[4])) {
                $temp = explode(' ', $line[4]);
                $line[2] = array_pop($temp);
                $line[3] = implode(' ', $temp);
            }
            $data['produit_appellation_id'] = $this->searchInfosProduit('appellation', $line[2]);
            $data['produit_classification_id'] = $this->searchInfosProduit('classification', $line[3]);

            $data['produit_etat_id'] = $this->searchInfosProduit('etat', $line[10], array('label' => $line[11]));
            $data['produit_format_id'] = $this->searchInfosProduit('format', $line[9], array('label' => $line[6], 'cl' => $line[8]));
            $data['produit_couleur_id'] = $this->searchInfosProduit('couleur', $line[14], array('label' => $line[13]));

            $data['produit_millesime'] = $line[12];

            $data['produit_best_after'] = date('Y', strtotime($line[36]));
            $data['produit_best_before'] = date('Y', strtotime($line[37]));

            $data['produit_type_id'] = 1;

            $data['taxe_id'] = $taxeId;

            $data['produit_statut_id'] = 1;
            $data['langue_id'] = 1;
            $result = $this->matchProduct($line);

            if (!empty($result)) {
                $data['produit_id'] = $result['produit_id'];
                $newProd = false;
            } else {
                $newProd = true;
            }

            $data['produit_id'] = $this->sql->save('produit', $data);
            $this->sql->save('produit_nom', $data);

            if ($newProd) {
                $boutique->createStatut('produit', $data['produit_id']);
                $this->write('add produit ' . $data['produit_id'] . ' / ligne ' . $i);
            } else {
                $this->write('update produit ' . $data['produit_id'] . ' / ligne ' . $i);
            }

            if(!empty($data['produit_id'])) {

                if (!empty($line[15])) {
                    $this->produitCategorie($line, $data);
                }

                // TODO : description du produit
                // $this->description($line, $data);

                if (!empty($line[26])) {
                    $this->caracteristique(Caracteristique::NOTE_PARKER, $line[26], $data);
                }


                // création des attributs pour les conditionnements => variations de prix
                $dataAtt = array('produit_id' => $data['produit_id']);
                $dataAtt['produit_attribut_statut_id'] = ($line[21] == 1) ? 1 : 2;
                $dataAtt['produit_attribut_prix'] = $line[23];
                $dataAtt['produit_attribut_achat_der'] = $line[25];
                $dataAtt['produit_attribut_achat'] = $line[24];
                $dataAtt['produit_attribut_conditionnement'] = $line[20];

                $dataAtt['produit_attribut_quantite'] = $line[17];
                $dataAtt['produit_attribut_quantite_p'] = $line[17];

                $att = $this->matchAttribut($line, $data);
                if (!empty($att)) {
                    $dataAtt['produit_attribut_id'] = $att['produit_attribut_id'];
                    $newAtt = false;
                } else {
                    $newAtt = true;
                }

                $dataAtt['produit_attribut_id'] = $this->sql->save('produit_attribut', $dataAtt);

                if ($newAtt) {
                    $this->write('add attribut ' . $dataAtt['produit_attribut_id'] . ' / ligne ' . $i);
                } else {
                    $this->write('update attribut ' . $dataAtt['produit_attribut_id'] . ' / ligne ' . $i);
                }
                //-- FIN Attribut

                // mise a jour stock global du produit
                $stock->updateProdWithAtt($data['produit_id']);
            } else {
                $this->write('ligne ' . $i, 'error');
            }

            /*if ($i == 100) {
                echo 'fin import produit' . "\n";
                exit;
            }*/
        }
    }


    public function images($lines, $formvars = array())
    {
        // TODO
    }

    /**
     * @param $message
     * @param string $type : info | error | dump
     */
    protected function write($message, $type = 'info')
    {
        if ($type == 'dump') {
            dump($message);
        } else {
            if ($this->output instanceof OutputInterface) {
                $this->output->writeln('<' . $type . '>' . $message . '</' . $type . '>');
            } else {
                echo $message . "\n";
            }
        }
    }

    public static function getTablesJoin($key){
        if(!isset(self::$tablesJoin[$key])){
            throw new DediException('Tables join '.$key.' not found !');
        }
        $tablesJoin = array();
        if(!empty(self::$tablesJoin[$key])){
            foreach(self::$tablesJoin[$key] as $table){
                if(isset(self::$tables[$table])){
                    $tablesJoin = array_merge($tablesJoin, self::$tables[$table]);
                }
            }
        }
        return $tablesJoin;
    }
}

