<?php

namespace Controller;

use Breadcrumb;
use Client;
use Panier;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Adresse;
use Livraison;
use Paiement;
use Texte;
use Tpe;
use Commande;
use DediException;
use Avoir;
use BonAchat;
use Pays;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class CoordonneesController
 * @package Controller
 */
class CoordonneesController extends Controller {

    /**
     * @Route(
     *      path         = "/checkout_shipping.html",
     *      name         = "coordonnees",
     *      methods      = { "GET" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function show()
    {
        $panier = Panier::getInstance();
        $adresse = new Adresse();
        $livraison = new Livraison();
        $client = $this->getCurrentClient();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();

        $addrDefault = $adresse->getDefault($client->client_id);

        $shipping = $panier->getShipping();

        // init addresse
        if (!empty($addrDefault)) {
            $addrShip = $panier->getAddressShipping();
            if (empty($addrShip)) {
                $panier->setAddressShipping($addrDefault);
            }

            $addrBill = $panier->getAddressBilling();
            if (empty($addrBill)) {
                $panier->setAddressBilling($addrDefault);
            }
        }

        $addresses = array(
            'shipping' => $panier->getAddressShipping(),
            'billing' => $panier->getAddressBilling(),
        );
        $weight = $panier->getWeight();

        $paramLivr = array();
        $paramLivr['pays_id'] = $addresses['shipping']->pays_id;
        $paramLivr['poids'] = $weight;
        $paramLivr['prix'] = $sTotalHT;

        if ($panier->isTva()) {
            $paramLivr['prix_ttc'] = $sTotalTTC;
        } else {
            $paramLivr['prix_ht'] = $sTotalHT;
        }
        $livraisons = $livraison->get($paramLivr);

        $tpl_vars['livraisons'] = $livraisons;
        $tpl_vars['port'] = (!empty($shipping)) ? $shipping->toArray() : array();

        $flashBag = $this->getFlashBag();
        if ($flashBag->has('error')) {
            $tpl_vars['error'] = $flashBag->get('error');
        }

        return $this->render('coordonnees.tpl', $tpl_vars);
    }

    /**
     * @Route(
     *      path         = "/checkout_shipping.html",
     *      name         = "coordonnees_post",
     *      methods      = { "POST" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function post()
    {

        $flashBag = $this->getFlashBag();
        $texte = new Texte();
        $panier = Panier::getInstance();

        if ($this->request->get('livraison_zone_id') == false) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errSelectLivraison', 'Vous devez selectionner un mode de livraison')
            );
        }

        $addrShipping = $panier->getAddressShipping();
        if (! $addrShipping instanceof Adresse || empty($addrShipping->adresse_id)) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errAddrLivraison', 'Vous devez selectionner une adresse de livraison')
            );
        }

        if ($flashBag->has('error') == false) {
            $url = $this->generateUrl('recapitulatif');
        } else {
            $url = $this->generateUrl('coordonnees');
        }

        return $this->redirect($url);
    }
}
