<?php

use Symfony\Component\Routing\Exception\RouteNotFoundException;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Routing\Loader\AnnotationClassLoader;
use Symfony\Component\Routing\Router as RouterSF;
use Symfony\Component\Routing\RequestContext;
use Symfony\Component\Routing\Loader\AnnotationDirectoryLoader;

/**
 * Class Router
 */
class Router implements UrlGeneratorInterface
{
    /**
     * @var \Symfony\Component\Routing\Router
     */
    protected $router;

    /**
     * @var AnnotationClassLoader
     */
    protected $annotationClassLoader;

    /**
     * @var AnnotationDirectoryLoader
     */
    protected $annotationDirectoryLoader;


    /**
     * Router constructor.
     *
     * @param AnnotationClassLoader $annotationClassLoader
     * @param AnnotationDirectoryLoader                               $annotationDirectoryLoader
     * @param string                                                  $resource
     */
    public function __construct(AnnotationClassLoader $annotationClassLoader, AnnotationDirectoryLoader $annotationDirectoryLoader, $resource = "src/Dedi/Controller")
    {
        $requestUri = (!empty($_SERVER['REQUEST_URI'])) ? $_SERVER['REQUEST_URI'] : '';
        $requestContext = new RequestContext($requestUri);

        $this->annotationClassLoader = $annotationClassLoader;
        $this->annotationDirectoryLoader = $annotationDirectoryLoader;

        $cacheDir = SITE_DIR . 'storage' . DS . 'php' . DS . 'router';

        if (is_dir($cacheDir) == false) {
            mkdir($cacheDir, 0775, true);
            chmod($cacheDir, 0775);
        }

        $options = array();
        if (getenv('APP_CACHE')) {
            $options['cache_dir'] = $cacheDir;
        }

        $this->router = new RouterSF(
            $this->annotationDirectoryLoader,
            $resource,
            $options,
            $requestContext
        );
    }

    /**
     * {@inheritdoc}
     * @throws RouteAndI18nRouteNotFoundException
     */
    public function generate($name, $parameters = array(), $referenceType = RouterSF::ABSOLUTE_PATH)
    {
        $router = $this->getRouter();

        try {
            return $router->generate($name, $parameters, $referenceType);
        } catch (RouteNotFoundException $routeNotFoundException) {
            $locale = !empty($parameters['_locale'])
                ? $parameters['_locale']
                : $router->getContext()->getParameter('_locale')
            ;

            $i18nName = sprintf('%s_%s', $name, $locale);

            try {
                return $router->generate($i18nName, $parameters, $referenceType);
            } catch (RouteNotFoundException $i18nRouteNotFoundException) {
                throw new RouteAndI18nRouteNotFoundException(
                    sprintf("Pas de route %s ni %s", $name, $i18nName),
                    0,
                    $routeNotFoundException
                );
            }
        }
    }

    /**
     * retourne le Router
     *
     * @return \Symfony\Component\Routing\Router
     */
    public function getRouter()
    {
        return $this->router;
    }

    /**
     * @return AnnotationClassLoader
     */
    public function getAnnotationClassLoader()
    {
        return $this->annotationClassLoader;
    }

    /**
     * @return AnnotationDirectoryLoader
     */
    public function getAnnotationDirectoryLoader()
    {
        return $this->annotationDirectoryLoader;
    }

    /**
     * {@inheritdoc}
     */
    public function setContext(RequestContext $context)
    {
        return $this->getRouter()->setContext($context);
    }

    /**
     * {@inheritdoc}
     */
    public function getContext()
    {
        return $this->getRouter()->getContext();
    }
}
