<?php

namespace Shopbydedi;

use Boutique;
use Caracteristique;
use CaracteristiqueValeur;
use Categorie;
use Gamme;
use Marque;
use Symfony\Component\HttpFoundation\Request;
use Texte;
use Tools;

/**
 * Class Listing
 */
class Listing
{

    /**
     * @var array
     */
    protected $products;
    /**
     * @var CaracteristiqueValeur[]
     */
    protected $caracteristiques;
    /**
     * @var array
     */
    protected $caraProducts;
    /**
     * @var Marque[]
     */
    protected $marques;
    /**
     * @var Gamme[]
     */
    protected $gammes;
    /**
     * @var Categorie[]
     */
    protected $categories;
    /**
     * @var string
     */
    protected $search;
    /**
     * @var array
     */
    protected $prix;
    /**
     * @var int
     */
    protected $page;
    /**
     * @var int
     */
    protected $limit;
    /**
     * @var string
     */
    protected $group;
    /**
     * @var int
     */
    protected $prixmax;
    /**
     * @var int
     */
    protected $prixmin;
    /**
     * @var string
     */
    protected $order;
    /**
     * @var string
     */
    protected $caracteristiqueUrl;
    /**
     * @var array
     */
    protected $statut;

    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    protected $request;

    /**
     *
     */
    public function __construct()
    {
        $this->reset();
    }

    /**
     * init the object
     */
    public function reset()
    {
        $config = Boutique::getInstance()->getConfig();

        $this->page = 1;
        $this->limit = (!empty($config['listing']['limit'])) ? $config['listing']['limit'] : 16;
        $this->order = (!empty($config['listing']['order'])) ? $config['listing']['order'] : '';
        $this->group = '';
        $this->marques = array();
        $this->resetCategories();
        $this->gammes = array();
        $this->resetProducts();
        $this->caraProducts = array();
        $this->caracteristiques = array();
        $this->search = null;
        $this->prix = array();
        $this->statut = array();
        $this->request = null;

        return $this;
    }

    /**
     * @return $this
     */
    public function resetCategories()
    {
        $this->categories = array();
        return $this;
    }
    /**
     * @return $this
     */
    public function resetProducts()
    {
        $this->products = array();
        return $this;
    }

    /**
     * @param Request $request
     * @return $this
     */
    public function setRequest(Request $request)
    {
        $this->request = $request;
        return $this;
    }

    /**
     * @param int $statut
     * * @return $this
     */
    public function addStatut($statut)
    {
        $this->statut[] = $statut;
        return $this;
    }

    /**
     * @param Categorie $categorie
     * * @return $this
     */
    public function addCategorie(Categorie $categorie)
    {
        $this->categories[] = $categorie;
        return $this;
    }

    /**
     * @param Gamme $gamme
     * * @return $this
     */
    public function addGamme(Gamme $gamme)
    {
        $this->gammes[] = $gamme;
        return $this;
    }

    /**
     * @param Marque $marque
     * * @return $this
     */
    public function addMarque(Marque $marque)
    {
        $this->marques[] = $marque;
        return $this;
    }

    /**
     * @param CaracteristiqueValeur $caracteristique
     * * @return $this
     */
    public function addCaracteristique(CaracteristiqueValeur $caracteristique)
    {
        $this->caracteristiques[] = $caracteristique;
        return $this;
    }

    /**
     * @param $search string
     * * @return $this
     */
    public function setSearch($search)
    {
        $this->search = $search;
        return $this;
    }

    /**
     * @param $page int
     * * @return $this
     */
    public function setPage($page)
    {
        $this->page = $page;
        return $this;
    }


    /**
     * @param $order string
     * @return $this
     */
    public function setOrder($order)
    {
        $this->order = $order;
        return $this;
    }

    /**
     * @param $limit int
     * @return $this
     */
    public function setLimit($limit)
    {
        $this->limit = $limit;
        return $this;
    }

    /**
     * @param $group string
     * @return $this
     */
    public function setGroup($group)
    {
        $this->group = $group;
        return $this;
    }


    /**
     * @param $prix
     */
    public function setPrix($prix)
    {
        $this->prix = $prix;
    }

    /**
     * @return int
     */
    public function getPrixMax()
    {
        return $this->prixmax;
    }

    /**
     * @return int
     */
    public function getPrixMin()
    {
        return $this->prixmin;
    }

    /**
     * @param $arrProd
     */
    public function setPrice($arrProd)
    {

        $lepluspetitprix = (!empty($arrProd[0])) ? round($arrProd[0]['produit_prix_ttc']) : 0;
        $leplusgrandprix = (!empty($arrProd[0])) ? round($arrProd[0]['produit_prix_ttc']) : 0;

        foreach ($arrProd as $oneProd) {
            if ($oneProd['produit_prix_ttc'] > $leplusgrandprix) {
                $leplusgrandprix = round($oneProd['produit_prix_ttc']) + 1;
            } elseif ($oneProd['produit_prix_ttc'] < $lepluspetitprix) {
                $lepluspetitprix = round($oneProd['produit_prix_ttc']) - 1;
            }
        }
        if ($lepluspetitprix < 0) {
            $lepluspetitprix=0;
        }
        if ($leplusgrandprix < 0) {
            $leplusgrandprix=0;
        }
        if (!isset($_GET['prix_max'])) {
             $_SESSION['prixmax'] = $leplusgrandprix;
             $this->prixmax = $leplusgrandprix ;
        } else {
            $this->prixmax = $_SESSION['prixmax'];
        }
        if (!isset($_GET['prix_min'])) {
             $_SESSION['prixmin'] = $lepluspetitprix;
             $this->prixmin = $lepluspetitprix ;
        } else {
            $this->prixmin = $_SESSION['prixmin'];
        }
    }

    /**
     * @return array
     *
     * function to get all products match with the criteria add : Categorie / Gamme / Marque / etc...
     */

    public function getProducts()
    {
        if (empty($this->products)) {
            $this->loadProducts();
        }

        return $this->products;
    }

    /**
     *
     * @return array retourne un tableau avec les gammes des produits de la selection
     */
    public function getGammes()
    {
        if (empty($this->products)) {
            $this->loadProducts();
        }

        $gammesId = Tools::extractValue('gamme_id', $this->products);


        $result = array();
        if (!empty($gammesId)) {
            $gammesId = array_unique($gammesId);
            $gammesId = array_filter($gammesId);

            $gamme = new Gamme();
            $result = $gamme->get(array('gamme_id' => $gammesId));
        }

        return $result;
    }

    /**
     *
     * @return array retourne un tableau avec les marques des produits de la selection
     */
    public function getMarques()
    {
        if (empty($this->products)) {
            $this->loadProducts();
        }

        // @TODO
    }

    /**
     *
     * @return array retourne un tableau avec les marques des produits de la selection
     */
    public function getCategorie()
    {
        return $this->categories;
    }

    /**
     * function to get just the product to display on the page, use page and limit
     *
     * @param array $param
     *
     * @return array
     *
     */
    public function getProductsToDisplay($param = array())
    {
        $produit = new Produit();

        if (empty($this->products)) {
            $this->loadProducts();
        }

        $offset = $this->limit * ($this->page - 1);
        $productsSlice = array_slice($this->products, $offset, $this->limit);

        $productsToDisplay = array();

        if (!empty($productsSlice)) {
            if (!empty($this->statut)) {
                $param['statut'] = $this->statut;
            }

            $param["produit_id"] = Tools::extractValue("produit_id", $productsSlice);
            $param["produit_attribut_id"] = Tools::extractValue("produit_attribut_id", $productsSlice);
            $param["order_by_result"] = true;

            $productsToDisplay = $produit->get($param);
        }

        return $productsToDisplay;
    }

    /**
     * @return int
     */
    public function getCountProducts()
    {
        return count($this->products);
    }

    public function getPagination()
    {
        $pagination = array();

        $total = $this->getCountProducts();
        $first = ($this->page - 1) * $this->limit + 1;
        if ($first > $total) {
            $first = $total;
        }
        $last = ($this->page) * $this->limit;
        if ($last > $total) {
            $last = $total;
        }
        $pages = ceil($total / $this->limit);

        $pas = 10;
        $tronc = floor($this->page / $pas);

        if ($tronc == 0) {
            $pagination['INTMIN'] = 1;
            $pagination['INTMAX'] = 9;
            $pagination['PAS'] = $pas - 1;
        } else {
            $pagination['INTMIN'] = $tronc * $pas;
            $pagination['INTMAX'] = ($tronc * $pas) + $pas;
            $pagination['PAS'] = $pas;
        }

        // Affect des donnees de pagination
        $pagination['NBROW'] = $total;
        $pagination['FIRSTROW'] = $first;
        $pagination['LASTROW'] = $last;
        $pagination['NBPAGE'] = $pages;
        $pagination['CURRENTPAGE'] = ($this->page > $pages) ? $pages : $this->page;

        if ($total > 0) {
            $pagination['start'] = ($pagination['CURRENTPAGE'] - 3 < 1) ? 1 : $pagination['CURRENTPAGE'] - 3;
            $pagination['end'] = ($pagination['start'] + 5 > $pagination['NBPAGE']) ? $pagination['NBPAGE'] : $pagination['start'] + 5;
        } else {
            $pagination['start'] = 1;
            $pagination['end'] = 1;
        }

        $pagination['prec'] = $pagination['CURRENTPAGE'] - 1;
        $pagination['suiv'] = $pagination['CURRENTPAGE'] + 1;

        return $pagination;
    }

    /**
     * @return array
     */
    public function getCaracteristiqueSelect()
    {
        $arrCara = array();
        if (!empty($this->caracteristiques)) {
            $caracteristique = new Caracteristique();

            $url = (!empty($this->caracteristiqueUrl)) ?
                str_replace('/' . $this->caracteristiqueUrl, '', $this->request->getRequestUri()) :
                $this->request->getRequestUri();

            foreach ($this->caracteristiques as $k => $cara) {
                $caraTmp = array($cara->toArray());
                $caracteristique->setUrl($caraTmp, $this->caracteristiques, $url);
                $arrCara[] = $caraTmp[0];
            }
        }

        return $arrCara;
    }

    /**
     * @return array
     *
     * function to get all cara match with the product in the listing
     */
    public function getCaracteristique($caracteristiqueId = null)
    {
        if (empty($this->caraProducts)) {
            $this->loadCaracteristique();
        }

        if (!is_null($caracteristiqueId)) {

            if (!is_array($caracteristiqueId)) {
                $caracteristiqueId = array($caracteristiqueId);
            }

            $caracs = array_map(
                function ($value) use ($caracteristiqueId) {
                    if (in_array($value['caracteristique_id'], $caracteristiqueId)) {
                        return $value;
                    }
                    return null;
                },
                $this->caraProducts
            );
            return array_filter($caracs);
        }

        return $this->caraProducts;
    }

    /**
     * @param $url
     */
    public function setCaracteristiqueUrl($url)
    {
        $this->caracteristiqueUrl = $url;

        $caracteristique = new Caracteristique();

        $arrCara = $caracteristique->parseUrl($this->caracteristiqueUrl);

        foreach ($arrCara as $caraId) {
            $this->addCaracteristique(new CaracteristiqueValeur($caraId));
        }
    }

    /**
     * function to load all products match with the criteria add : Categorie / Gamme / Marque / etc...
     *
     * @param array $param
     *
     */
    protected function loadProducts($param = array())
    {
        $produit = new Produit();

        $param['noBloc'] = true;
        if (!empty($this->marques)) {
            foreach ($this->marques as $m) {
                $param['marque'][] = $m->marque_id;
            }
        }

        if (!empty($this->categories)) {
            $param['categorie_ext_id'] = $this->categories[0]->categorie_id;
        }

        if (!empty($this->gammes)) {
            foreach ($this->gammes as $g) {
                $param['gamme'][] = $g->gamme_id;
            }
        }

        if (!empty($this->caracteristiques)) {
            foreach ($this->caracteristiques as $c) {
                $param['caracteristique'][] = $c->caracteristique_valeur_id;
            }
        }

        if (!empty($this->prix)) {
            $param['prix'] = $this->prix;
        }

        if (is_null($this->search) == false) {
            if (is_array($this->search)) {
                list($idx, $search) = $this->search;
            } else {
                $idx = 'motclef';
                $search = $this->search;
            }
            $param[$idx] = $search;
        }

        if (!empty($this->statut)) {
            $param['statut'] = $this->statut;
        }

        $param['order'] = $this->order;
        $param['group'] = $this->group;

        $this->products = $produit->getSearch($param);
    }

    /**
     *
     * function to load the list of the cara for the products
     */
    protected function loadCaracteristique()
    {
        $caracteristique = new Caracteristique();

        $arrValeur = $caracteristique->getValeurProduit([
            'listProduit' => Tools::extractValue('produit_id', $this->products)
        ]);

        if (!empty($arrValeur)) {

            $url = (!empty($this->caracteristiqueUrl)) ?
                str_replace('/' . $this->caracteristiqueUrl, '', $this->request->getRequestUri()) :
                $this->request->getRequestUri();

            $caracteristique->setUrl($arrValeur, $this->caracteristiques, $url);
        }

        $this->caraProducts = $caracteristique->constructCara($arrValeur, $this->caracteristiques, $this->products);
    }

    /**
     * @return array
     */
    public function getReferencement()
    {
        $texte = new Texte();

        $ref = array(
            'title' => '',
            'description' => '',
            'keywords' => '',
            'other' => ''
        );

        if (!empty($this->categories)) {
            $type = 'categorie';
            $suffixeTxt = (empty($this->categories[0]->parent_id)) ? 'parent' : 'categorie';
            $obj = $this->categories[0];
            if (!empty($this->categories[0]->parent_id)) {
                $parent = new Categorie($this->categories[0]->parent_id);
                $obj->parent_nom_nom = $parent->categorie_nom_nom;
            }
        } elseif (!empty($this->marques)) {
            $type = $suffixeTxt = 'marque';
            $obj = $this->marques[0];
        } elseif (!empty($this->gammes)) {
            $type = $suffixeTxt = 'gamme';
            $obj = $this->gammes[0];
        } elseif (!empty($this->search)) {
            $ref['title'] = $texte->getOne('ref_title_search', true, array('motclef' => $this->search));
            $ref['description'] = $texte->getOne('ref_desc_search', true, array('motclef' => $this->search));
        }

        if (!empty($suffixeTxt) && !empty($obj)) {
            $ref['title'] = (!empty($obj->{$type . '_nom_meta_title'})) ? $obj->{$type . '_nom_meta_title'} : $texte->getOne('ref_title_' . $suffixeTxt, true, $obj->toArray());
            $ref['description'] = (!empty($obj->{$type . '_nom_meta_desc'})) ? $obj->{$type . '_nom_meta_desc'} : $texte->getOne('ref_desc_' . $suffixeTxt, true, $obj->toArray());
            $ref['keywords'] = (!empty($obj->{$type . '_nom_meta_kw'})) ? $obj->{$type . '_nom_meta_kw'} : $texte->getOne('ref_kw_' . $suffixeTxt, true, $obj->toArray());
            $ref['other'] = (!empty($obj->{$type . '_nom_meta_other'})) ? $obj->{$type . '_nom_meta_other'} : $texte->getOne('ref_other_' . $suffixeTxt, true, $obj->toArray());
        }

        return $ref;
    }



    /**
     * Get current cat and bring back parent complete url
     *
     * @return array
     */
    public function getCurrentCategory()
    {
        $found = array();
        $catId = $this->request->get("id");

        if (!empty($catId)) {
            $cat = new Categorie($catId);
            if (!is_null($cat->parent_id)) {
                $parent = new Categorie($cat->parent_id);
                $type = "liste_1";
                $param = array(
                    "id"            => $parent->categorie_id,
                    "categorie_nom" => Tools::parseURL($parent->categorie_nom_nom),
                );
                if (!is_null($parent->parent_id)) {
                    $gparent = new Categorie($parent->parent_id);
                    $param["parent_nom"] = Tools::parseURL($gparent->categorie_nom_nom);
                    $type = "liste_2";
                }
                $cat = $cat->toArray();
                $cat["url"] = \Application::getRouter()->generate($type, $param);
                $found = $cat;
            }
        }
        return $found;
    }

    /***
     * Bring back all child categories and assign its urls
     *
     * @return array|null
     */
    public function getChildCategories()
    {
        $found = array();
        $catId = $this->request->get("id");
        if (!empty($catId)) {
            $cat = new Categorie($catId);
            $children = $cat->get(array("parent_id" => $cat->categorie_id));

            // parent
            $param = array(
                "parent_nom" => Tools::parseURL($cat->categorie_nom_nom),
            );
            $type = "liste_2";

            // final urls
            $router = \Application::getRouter();
            foreach ($children as $k => $child) {
                $p = array_merge($param, array(
                    "id"            => $child["categorie_id"],
                    "categorie_nom" => Tools::parseURL($child["categorie_nom_nom"]),
                ));
                try {
                    $children[$k]["url"] = $router->generate($type, $p);
                } catch (\Exception $e) {

                }
            }

            $found = $children;
        }
        return $found;
    }
}
